/// <reference types="node" />
import type { ReducedHelperObject } from './helpers/reducer';
import type { ClientId, TargetId } from './targets/targets';
import type { Param, PostDataCommon, Request as NpmHarRequest } from 'har-format';
import type { UrlWithParsedQuery } from 'url';
export { availableTargets, extname } from './helpers/utils';
export { addTarget, addTargetClient } from './targets/targets';
/** is this wrong?  yes.  according to the spec (http://www.softwareishard.com/blog/har-12-spec/#postData) it's technically wrong since `params` and `text` are (by the spec) mutually exclusive.  However, in practice, this is not what is often the case.
 *
 * In general, this library takes a _descriptive_ rather than _perscriptive_ approach (see https://amyrey.web.unc.edu/classes/ling-101-online/tutorials/understanding-prescriptive-vs-descriptive-grammar/).
 *
 * Then, in addition to that, it really adds to complexity with TypeScript (TypeScript takes this constraint very very seriously) in a way that's not actually super useful.  So, we treat this object as though it could have both or either of `params` and/or `text`.
 */
type PostDataBase = PostDataCommon & {
    params?: Param[];
    text?: string;
};
export type HarRequest = Omit<NpmHarRequest, 'postData'> & {
    postData: PostDataBase;
};
export interface RequestExtras {
    allHeaders: ReducedHelperObject;
    cookiesObj: ReducedHelperObject;
    fullUrl: string;
    headersObj: ReducedHelperObject;
    postData: PostDataBase & {
        boundary?: string;
        jsonObj?: ReducedHelperObject;
        paramsObj?: ReducedHelperObject;
    };
    queryObj: ReducedHelperObject;
    uriObj: UrlWithParsedQuery;
}
export type Request = HarRequest & RequestExtras;
interface Entry {
    request: Partial<HarRequest>;
}
interface HarEntry {
    log: {
        creator: {
            name: string;
            version: string;
        };
        entries: Entry[];
        version: string;
    };
}
export interface HTTPSnippetOptions {
    harIsAlreadyEncoded?: boolean;
}
export declare class HTTPSnippet {
    requests: Request[];
    constructor(input: HarEntry | HarRequest, opts?: HTTPSnippetOptions);
    prepare: (harRequest: HarRequest, options: HTTPSnippetOptions) => {
        allHeaders: {
            [x: string]: string | string[];
        };
        fullUrl: string;
        url: string;
        uriObj: {
            query: ReducedHelperObject;
            search: string;
            path: string | null;
            auth: string | null;
            hash: string | null;
            host: string | null;
            hostname: string | null;
            href: string;
            pathname: string | null;
            protocol: string | null;
            slashes: boolean | null;
            port: string | null;
        };
        method: string;
        httpVersion: string;
        cookies: import("har-format").Cookie[];
        headers: import("har-format").Header[];
        queryString: import("har-format").QueryString[];
        headersSize: number;
        bodySize: number;
        comment?: string | undefined;
        postData: PostDataCommon & {
            params?: Param[] | undefined;
            text?: string | undefined;
        } & {
            boundary?: string | undefined;
            jsonObj?: ReducedHelperObject | undefined;
            paramsObj?: ReducedHelperObject | undefined;
        };
        cookiesObj: ReducedHelperObject;
        headersObj: ReducedHelperObject;
        queryObj: ReducedHelperObject;
    };
    convert: (targetId: TargetId, clientId?: ClientId, options?: any) => string | false | string[];
}
