"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.restsharp = void 0;
var code_builder_1 = require("../../../helpers/code-builder");
var escape_1 = require("../../../helpers/escape");
function title(s) {
    return s[0].toUpperCase() + s.slice(1).toLowerCase();
}
exports.restsharp = {
    info: {
        key: 'restsharp',
        title: 'RestSharp',
        link: 'http://restsharp.org/',
        description: 'Simple REST and HTTP API Client for .NET',
    },
    convert: function (_a) {
        var method = _a.method, fullUrl = _a.fullUrl, headersObj = _a.headersObj, cookies = _a.cookies, postData = _a.postData, uriObj = _a.uriObj;
        var _b = new code_builder_1.CodeBuilder(), push = _b.push, join = _b.join;
        var isSupportedMethod = ['GET', 'POST', 'PUT', 'DELETE', 'PATCH', 'HEAD', 'OPTIONS'].includes(method.toUpperCase());
        if (!isSupportedMethod) {
            return 'Method not supported';
        }
        push('using RestSharp;\n\n');
        push("var options = new RestClientOptions(\"".concat(fullUrl, "\");"));
        push('var client = new RestClient(options);');
        // The first argument is the sub-path to the base URL, given as the
        // constructor to RestClient; for our purposes we're just giving the entire
        // URL as the base path so it can be an empty string
        push('var request = new RestRequest("");');
        // If we have multipart form data, set this value. Setting the content-type header manually and then trying to add a mutlipart file parameter
        var isMultipart = postData.mimeType && postData.mimeType === 'multipart/form-data';
        if (isMultipart) {
            push('request.AlwaysMultipartFormData = true;');
        }
        // Add headers, including the cookies
        Object.keys(headersObj).forEach(function (key) {
            // if we have post data, restsharp really wants to set the contentType
            // itself; do not add a content-type header or you end up with failures
            // which manifest as unhandled exceptions.
            //
            // The only case where we _do_ want to add it is if there's no postData
            // text, in which case there will be no `AddJsonBody` call, and restsharp
            // won't know to set the content type
            if (postData.mimeType && key.toLowerCase() === 'content-type' && postData.text) {
                if (isMultipart && postData.boundary) {
                    push("request.FormBoundary = \"".concat(postData.boundary, "\";"));
                }
                return;
            }
            push("request.AddHeader(\"".concat(key, "\", \"").concat((0, escape_1.escapeForDoubleQuotes)(headersObj[key]), "\");"));
        });
        cookies.forEach(function (_a) {
            var name = _a.name, value = _a.value;
            push("request.AddCookie(\"".concat(name, "\", \"").concat((0, escape_1.escapeForDoubleQuotes)(value), "\", \"").concat(uriObj.pathname, "\", \"").concat(uriObj.host, "\");"));
        });
        switch (postData.mimeType) {
            case 'multipart/form-data':
                if (!postData.params)
                    break;
                postData.params.forEach(function (param) {
                    if (param.fileName) {
                        push("request.AddFile(\"".concat(param.name, "\", \"").concat(param.fileName, "\");"));
                    }
                    else {
                        push("request.AddParameter(\"".concat(param.name, "\", \"").concat(param.value, "\");"));
                    }
                });
                break;
            case 'application/x-www-form-urlencoded':
                if (!postData.params)
                    break;
                postData.params.forEach(function (param) {
                    push("request.AddParameter(\"".concat(param.name, "\", \"").concat(param.value, "\");"));
                });
                break;
            case 'application/json': {
                if (!postData.text)
                    break;
                var text = JSON.stringify(postData.text);
                push("request.AddJsonBody(".concat(text, ", false);"));
                break;
            }
            default:
                if (!postData.text)
                    break;
                push("request.AddStringBody(\"".concat(postData.text, "\", \"").concat(postData.mimeType, "\");"));
        }
        push("var response = await client.".concat(title(method), "Async(request);\n"));
        push('Console.WriteLine("{0}", response.Content);\n');
        return join();
    },
};
