"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.curl = void 0;
var code_builder_1 = require("../../../helpers/code-builder");
var headers_1 = require("../../../helpers/headers");
var shell_1 = require("../../../helpers/shell");
/**
 * This is a const record with keys that correspond to the long names and values that correspond to the short names for cURL arguments.
 */
var params = {
    'http1.0': '0',
    'url ': '',
    cookie: 'b',
    data: 'd',
    form: 'F',
    globoff: 'g',
    header: 'H',
    insecure: 'k',
    request: 'X',
};
var getArg = function (short) { return function (longName) {
    if (short) {
        var shortName = params[longName];
        if (!shortName) {
            return '';
        }
        return "-".concat(shortName);
    }
    return "--".concat(longName);
}; };
exports.curl = {
    info: {
        key: 'curl',
        title: 'cURL',
        link: 'http://curl.haxx.se/',
        description: 'cURL is a command line tool and library for transferring data with URL syntax',
    },
    convert: function (_a, options) {
        var _b;
        var fullUrl = _a.fullUrl, method = _a.method, httpVersion = _a.httpVersion, headersObj = _a.headersObj, allHeaders = _a.allHeaders, postData = _a.postData;
        if (options === void 0) { options = {}; }
        var _c = options.indent, indent = _c === void 0 ? '  ' : _c, _d = options.short, short = _d === void 0 ? false : _d, _e = options.binary, binary = _e === void 0 ? false : _e, _f = options.globOff, globOff = _f === void 0 ? false : _f;
        // In the interest of having nicer looking snippets JSON should be indented separately from the
        // main command argument indentation.
        var indentJSON = '  ';
        var _g = new code_builder_1.CodeBuilder(__assign(__assign({}, (typeof indent === 'string' ? { indent: indent } : {})), { join: indent !== false ? " \\\n".concat(indent) : ' ' })), push = _g.push, join = _g.join;
        var arg = getArg(short);
        var formattedUrl = (0, shell_1.quote)(fullUrl);
        push("curl ".concat(arg('request'), " ").concat(method));
        if (globOff) {
            formattedUrl = unescape(formattedUrl);
            push(arg('globoff'));
        }
        push("".concat(arg('url ')).concat(formattedUrl));
        if (httpVersion === 'HTTP/1.0') {
            push(arg('http1.0'));
        }
        if ((0, headers_1.getHeader)(allHeaders, 'accept-encoding')) {
            // note: there is no shorthand for this cURL option
            push('--compressed');
        }
        // if multipart form data, we want to remove the boundary
        if (postData.mimeType === 'multipart/form-data') {
            var contentTypeHeaderName = (0, headers_1.getHeaderName)(headersObj, 'content-type');
            if (contentTypeHeaderName) {
                var contentTypeHeader = headersObj[contentTypeHeaderName];
                if (contentTypeHeaderName && contentTypeHeader) {
                    // remove the leading semi colon and boundary
                    // up to the next semi colon or the end of string
                    var noBoundary = String(contentTypeHeader).replace(/; boundary.+?(?=(;|$))/, '');
                    // replace the content-type header with no boundary in both headersObj and allHeaders
                    headersObj[contentTypeHeaderName] = noBoundary;
                    allHeaders[contentTypeHeaderName] = noBoundary;
                }
            }
        }
        // construct headers
        Object.keys(headersObj)
            .sort()
            .forEach(function (key) {
            var header = "".concat(key, ": ").concat(headersObj[key]);
            push("".concat(arg('header'), " ").concat((0, shell_1.quote)(header)));
        });
        if (allHeaders.cookie) {
            push("".concat(arg('cookie'), " ").concat((0, shell_1.quote)(allHeaders.cookie)));
        }
        // construct post params
        switch (postData.mimeType) {
            case 'multipart/form-data':
                (_b = postData.params) === null || _b === void 0 ? void 0 : _b.forEach(function (param) {
                    var post = '';
                    if (param.fileName) {
                        post = "".concat(param.name, "=@").concat(param.fileName);
                    }
                    else {
                        post = "".concat(param.name, "=").concat(param.value);
                    }
                    push("".concat(arg('form'), " ").concat((0, shell_1.quote)(post)));
                });
                break;
            case 'application/x-www-form-urlencoded':
                if (postData.params) {
                    postData.params.forEach(function (param) {
                        var unencoded = param.name;
                        var encoded = encodeURIComponent(param.name);
                        var needsEncoding = encoded !== unencoded;
                        var name = needsEncoding ? encoded : unencoded;
                        var flag = binary ? '--data-binary' : "--data".concat(needsEncoding ? '-urlencode' : '');
                        push("".concat(flag, " ").concat((0, shell_1.quote)("".concat(name, "=").concat(param.value))));
                    });
                }
                else {
                    push("".concat(binary ? '--data-binary' : arg('data'), " ").concat((0, shell_1.quote)(postData.text)));
                }
                break;
            default:
                // raw request body
                if (!postData.text) {
                    break;
                }
                // eslint-disable-next-line no-case-declarations -- builtPayload is only used here.
                var builtPayload = false;
                // If we're dealing with a JSON variant, and our payload is JSON let's make it look a little
                // nicer.
                if ((0, headers_1.isMimeTypeJSON)(postData.mimeType)) {
                    // If our postData is less than 20 characters, let's keep it all on one line so as to not
                    // make the snippet overly lengthy
                    if (postData.text.length > 20) {
                        try {
                            var jsonPayload = JSON.parse(postData.text);
                            // If the JSON object has a single quote we should prepare it inside of a HEREDOC
                            // because the single quote in something like `string's` can't be escaped when used
                            // with `--data`.
                            //
                            // Basically this boils down to `--data @- <<EOF...EOF` vs `--data '...'`.
                            builtPayload = true;
                            if (postData.text.indexOf("'") > 0) {
                                push("".concat(binary ? '--data-binary' : arg('data'), " @- <<EOF\n").concat(JSON.stringify(jsonPayload, null, indentJSON), "\nEOF"));
                            }
                            else {
                                push("".concat(binary ? '--data-binary' : arg('data'), " '\n").concat(JSON.stringify(jsonPayload, null, indentJSON), "\n'"));
                            }
                        }
                        catch (err) {
                            // no-op
                        }
                    }
                }
                if (!builtPayload) {
                    push("".concat(binary ? '--data-binary' : arg('data'), " ").concat((0, shell_1.quote)(postData.text)));
                }
        }
        return join();
    },
};
