import type { Request } from '..';
import type { CodeBuilderOptions } from '../helpers/code-builder';
import type { Merge } from 'type-fest';
export type TargetId = keyof typeof targets;
export type ClientId = string;
export interface ClientInfo {
    description: string;
    key: ClientId;
    link: string;
    title: string;
}
export type Converter<T extends Record<string, any>> = (request: Request, options?: Merge<CodeBuilderOptions, T>) => string;
export interface Client<T extends Record<string, any> = Record<string, any>> {
    convert: Converter<T>;
    info: ClientInfo;
}
export type Extension = `.${string}` | null;
export interface TargetInfo {
    cli?: string;
    default: string;
    extname: Extension;
    key: TargetId;
    title: string;
}
export interface Target {
    clientsById: Record<ClientId, Client>;
    info: TargetInfo;
}
export declare const targets: {
    c: Target;
    clojure: Target;
    csharp: Target;
    go: Target;
    http: Target;
    java: Target;
    javascript: Target;
    json: Target;
    kotlin: Target;
    node: Target;
    objc: Target;
    ocaml: Target;
    php: Target;
    powershell: Target;
    python: Target;
    r: Target;
    ruby: Target;
    shell: Target;
    swift: Target;
};
export declare const isTarget: (target: Target) => target is Target;
export declare const addTarget: (target: Target) => void;
export declare const isClient: (client: Client) => client is Client<Record<string, any>>;
export declare const addTargetClient: (targetId: TargetId, client: Client) => void;
